package com.futu.course.common.intercepter;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.futu.course.common.annotation.NonLoginRequired;
import com.futu.course.common.exception.RRException;
import com.futu.course.common.utils.RedisUtil;
import com.futu.course.common.utils.StringUtils;
import com.futu.course.common.utils.TokenUtils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

@Component
@Slf4j
public class AuthIntercepter implements HandlerInterceptor {

    @Autowired
    protected RedisTemplate redisTemplate;

    @Autowired
    private RedisUtil redisUtil;

    public final static String ACCESSTOKEN = "token";

    public AuthIntercepter() {

    }

    public AuthIntercepter(RedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        //System.out.println("preHandle");

        String token = request.getHeader(ACCESSTOKEN);
//        String time = request.getHeader("time");
        log.info(request.getRequestURI());
        //log.info("deviceId={},token={},time={}",deviceId,token,time);
        try {
            if (!(handler instanceof HandlerMethod)) {
                return true;
            }
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            Method method = handlerMethod.getMethod();
            NonLoginRequired loginRequired = method.getAnnotation(NonLoginRequired.class);
            //有@LoginRequired注解，需要认证
            if (loginRequired != null) {
                return true;
            }
            log.info("---interceptertoken");
            log.info(token);
            if (StringUtils.isEmpty(token)) {
                log.info("*********************》token为空");
                throw new RRException("token错误,请重新登录", 101);
            }
//            if (StringUtils.isEmpty(time)) {
//                throw new RRException("时间错误", 103);
//            }

            log.info("调用token**********************》");
            log.info(token);
            log.info("************************************》");
            Long userId = TokenUtils.getUserId(token);
            log.info("userId={}",userId);

            Object obj = redisTemplate.opsForValue().get("user");
            if (obj == null) {
                log.info("************************用户token不存在");
                throw new RRException("token过期", 101);
            }



        } catch (RRException e) {
            log.info("调用了异常通知" + e.getMessage());
            throw e;
        }

        return true;
    }


}
